package com.terraformersmc.terrestria.feature.trees;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terrestria.feature.TreeDefinition;
import com.terraformersmc.terraform.util.Shapes;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2944;
import net.minecraft.class_3111;
import net.minecraft.class_3341;
import net.minecraft.class_3746;
import net.minecraft.class_3747;

public class CypressTreeFeature extends class_2944<class_3111> {
	private TreeDefinition.Basic tree;

	public CypressTreeFeature(Function<Dynamic<?>, ? extends class_3111> function, boolean notify, TreeDefinition.Basic tree) {
		super(function, notify);

		this.tree = tree;
	}

	public CypressTreeFeature sapling() {
		return new CypressTreeFeature(class_3111::method_13565, true, tree);
	}

	@Override
	public boolean method_12775(Set<class_2338> blocks, class_3747 world, Random rand, class_2338 origin, class_3341 boundingBox) {
		// Total tree height
		int height = rand.nextInt(5) + 12;

		// Maximum leaf radius.
		double maxRadius = 1.5 + 1.5 * rand.nextDouble();

		//If the tree can pass the max build height
		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		class_2338 below = origin.method_10074();
		if (!method_16430(world, below)) {
			return false;
		}

		if (!checkForObstructions(world, origin, height, (int) Math.ceil(maxRadius))) {
			return false;
		}

		method_12773(blocks, world, origin.method_10074(), class_2246.field_10566.method_9564(), boundingBox);

		growTrunk(blocks, world, new class_2338.class_2339(origin), height, boundingBox);

		class_2338.class_2339 pos = new class_2338.class_2339(origin);
		growLeaves(blocks, world, pos, height, maxRadius, boundingBox);

		return true;
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height, int radius) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int dY = origin.method_10264(); dY < height; dY++) {
			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		pos.method_10103(origin.method_10263(), origin.method_10264() + height, origin.method_10260());

		for (int i = 0; i < 4; i++) {
			if (!method_16432(world, pos.method_10098(class_2350.field_11036))) {
				return false;
			}
		}

		return true;
	}

	// Grows the center trunk.
	private void growTrunk(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 pos, int height, class_3341 boundingBox) {
		for (int i = 0; i < (height * .6); i++) {
			method_12773(blocks, world, pos, tree.getLog(), boundingBox);
			pos.method_10098(class_2350.field_11036);
		}
	}

	private void growLeaves(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 pos, int height, double maxRadius, class_3341 boundingBox) {
		int x = pos.method_10263();
		int y = pos.method_10264();
		int z = pos.method_10260();
		double radius;

		for (int dy = 0; dy < height; dy++) {
			pos.method_10103(x, y + dy, z);

			radius = maxRadius * radiusFactor(dy, height);

			if (radius < 0) {
				continue;
			}

			Shapes.circle(pos, radius, position -> {
				if (class_2944.method_16420(world, pos)) {
					method_12773(blocks, world, pos, tree.getLeaves(), boundingBox);
				}
			});
		}
	}

	// Provides the factor to the radius, where x is a double from 0.0 to 1.0 that represents the progress along the trunk.
	private double radiusFactor(double x, double height) {
		x = x / height;

		// A 3rd-degree polynomial approximating the shape of a cypress tree - increasing rapidly, and then tapering off.
		return 6.25 * (x * x * x) - 12.5 * (x * x) + 6.25 * x;
	}
}
