package com.terraformersmc.terrestria.feature.trees;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terrestria.feature.TreeDefinition;
import com.terraformersmc.terrestria.feature.trees.components.Branches;
import com.terraformersmc.terrestria.feature.trees.components.SmallLogs;
import com.terraformersmc.terrestria.feature.trees.templates.JapaneseTreeFeature;
import com.terraformersmc.terraform.block.SmallLogBlock;
import com.terraformersmc.terraform.util.Shapes;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import java.util.function.Predicate;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2397;
import net.minecraft.class_2680;
import net.minecraft.class_2902;
import net.minecraft.class_2944;
import net.minecraft.class_3111;
import net.minecraft.class_3341;
import net.minecraft.class_3486;
import net.minecraft.class_3747;

public class SakuraTreeFeature extends JapaneseTreeFeature implements SmallLogs, Branches {
	private TreeDefinition.Sakura tree;
	private boolean worldGen;

	public SakuraTreeFeature(Function<Dynamic<?>, ? extends class_3111> function, boolean notify, TreeDefinition.Sakura tree) {
		super(function, notify, tree);

		this.tree = tree;
		this.worldGen = !notify;
	}

	public SakuraTreeFeature sapling() {
		return new SakuraTreeFeature(class_3111::method_13565, true, tree);
	}

	@Override
	public void placeGroundCover(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 origin, double maxRadius, Random rand, class_3341 boundingBox) {
		int x = origin.method_10263();
		int z = origin.method_10260();

		Shapes.circle(origin, maxRadius, pos -> {
			if (pos.method_10263() == x && pos.method_10260() == z) {
				return;
			}

			if (rand.nextInt(2) != 0) {
				return;
			}

			class_2338 top = world.method_8598(worldGen ? class_2902.class_2903.field_13194 : class_2902.class_2903.field_13202, pos);

			if (!class_2944.method_16424(world, top)) {
				return;
			}

			if (class_2944.method_16430(world, top.method_10074()) ||
					world.method_16358(top.method_10074(), state -> state.method_11618().method_15772().method_15791(class_3486.field_15517))) {
				method_12773(blocks, world, top, tree.getLeafPile(), boundingBox);
			}
		});
	}

	@Override
	public void placeBranch(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 pos, int length, class_2350 direction, class_3341 boundingBox) {
		for (int i = 0; i < length; i++) {
			pos.method_10098(direction);
			method_12773(blocks, world, pos, tree.getLog(), boundingBox);
		}
	}

	@Override
	public void correctLogStates(Set<class_2338> blocks, class_3747 world, class_3341 boundingBox) {
		for (class_2338 log : blocks) {
			boolean leaves = world.method_16358(log, tested -> tested.method_11614() instanceof SmallLogBlock && tested.method_11654(SmallLogBlock.HAS_LEAVES));

			Predicate<class_2680> tester = tested -> tested.method_11614() instanceof SmallLogBlock || (!leaves && tested.method_11614() instanceof class_2397) || tested.method_11619();

			method_12773(
					blocks,
					world,
					log,
					tree.getLog()
							.method_11657(SmallLogBlock.UP, world.method_16358(log.method_10084(), tester))
							.method_11657(SmallLogBlock.DOWN, world.method_16358(log.method_10074(), tester))
							.method_11657(SmallLogBlock.NORTH, world.method_16358(log.method_10095(), tester))
							.method_11657(SmallLogBlock.EAST, world.method_16358(log.method_10078(), tester))
							.method_11657(SmallLogBlock.SOUTH, world.method_16358(log.method_10072(), tester))
							.method_11657(SmallLogBlock.WEST, world.method_16358(log.method_10067(), tester))
							.method_11657(SmallLogBlock.HAS_LEAVES, leaves),
					boundingBox
			);
		}
	}

	@Override
	protected void tryPlaceLeaves(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 pos, class_3341 boundingBox) {
		if (world.method_16358(pos, candidate -> candidate.method_11614() instanceof SmallLogBlock)) {
			method_12773(blocks, world, pos, tree.getLogLeaves(), boundingBox);
		} else {
			super.tryPlaceLeaves(blocks, world, pos, boundingBox);
		}
	}
}
