package com.terraformersmc.terrestria.feature.trees.templates;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terraform.util.Shapes;
import com.terraformersmc.terrestria.feature.TreeDefinition;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2944;
import net.minecraft.class_3111;
import net.minecraft.class_3341;
import net.minecraft.class_3746;
import net.minecraft.class_3747;

public class ConiferTreeFeatureNew extends class_2944<class_3111> {
	private TreeDefinition.Basic tree;

	public ConiferTreeFeatureNew(Function<Dynamic<?>, ? extends class_3111> function, boolean notify, TreeDefinition.Basic tree) {
		super(function, notify);

		this.tree = tree;
	}

	public ConiferTreeFeatureNew sapling() {
		return new ConiferTreeFeatureNew(class_3111::method_13565, true, tree);
	}

	@Override
	public boolean method_12775(Set<class_2338> blocks, class_3747 world, Random rand, class_2338 origin, class_3341 boundingBox) {
		int height = getLeafHeight(rand);
		int bareTrunkHeight = getBareTrunkHeight(rand);
		int maxLeafRadius = getMaxLeafRadius(rand);
		int leafLayers = getLeafLayers(rand);
		double shrinkAmount = getShrinkAmount();
		int trunkRadius = getMaxTrunkRadius(rand);

		//If the tree doesn't have enough room with it's current height before build limit
		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		//If the ground below the sapling is good enough for a tree
		if (!method_16430(world, origin.method_10074())) {
			return false;
		}

		//If there is room for the tree
		if (!checkForObstructions(world, origin, height, bareTrunkHeight, maxLeafRadius)) {
			return false;
		}

		//Set the block below the trunk to dirt (because vanilla does it)
		method_12773(blocks, world, origin.method_10074(), class_2246.field_10566.method_9564(), boundingBox);
		growTrunk(blocks, world, new class_2338.class_2339(origin), height + bareTrunkHeight, trunkRadius, boundingBox, rand);
		growLeaves(blocks, world, new class_2338.class_2339(origin).method_10104(class_2350.field_11036, bareTrunkHeight), height, maxLeafRadius, shrinkAmount, leafLayers, boundingBox);

		return true;
	}

	private void growLeaves(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 pos, int height, int maxRadius, double shrinkAmmount, int layers, class_3341 boundingBox) {
		int x = pos.method_10263();
		int y = pos.method_10264();
		int z = pos.method_10260();
		int layerHeight = (height / layers);
		int leafLayerHeight = 2 * (height / layers);

		for (int layer = 0; layer < layers; layer++) {
			if (layer == (layers - 1)) { //Limit the top layer to not be too pointy
				leafLayerHeight = layerHeight;
			}
			for (int currentHeight = 0; currentHeight < leafLayerHeight; currentHeight++) {

				pos.method_10103(x, y + (layer * layerHeight) + currentHeight, z);

				Shapes.canopyCircle(pos,
						outerRadius(maxRadius - (layer * shrinkAmmount), currentHeight, leafLayerHeight),
						innerRadius(maxRadius - (layer * shrinkAmmount), currentHeight, layerHeight),
						position -> {
							if (class_2944.method_16420(world, pos)) {
								method_12773(blocks, world, pos, tree.getLeaves(), boundingBox);
							}
						});
			}
		}
	}

	/**
	 * @param maxRadius     the maximum radius for the leaves to generate
	 * @param currentHeight the progress from 0 to the height that the leaves have generated (the x value of the polynomial)
	 * @param height        the target height for the leaf layer
	 * @return the radius of the outside leaf layer at it's current y-height
	 */
	private double outerRadius(double maxRadius, double currentHeight, double height) {
		double x = currentHeight / height;
		// A 3rd-degree polynomial approximating the shape of a Conifer tree. from 0-1
		return maxRadius * ((-0.6 * (x * x * x) + 1.96 * (x * x) - 2.37 * x) + 1);
	}

	/**
	 * @param maxRadius     the maximum radius for the leaves to generate
	 * @param currentHeight the progress from 0 to the height that the leaves have generated (the x value of the polynomial)
	 * @param height        the target height for the leaf layer
	 * @return the radius of the inner leaf layer at it's current y-height
	 */
	private double innerRadius(double maxRadius, double currentHeight, double height) {
		double x = currentHeight / height;
		x = maxRadius * ((-3.24 * (x * x * x) + .25 * (x * x) - 2.98 * x) + 1) - 2.5;
		return x < 0 ? 0 : x;
	}

	public void growTrunk(Set<class_2338> blocks, class_3747 world, class_2338.class_2339 pos, int height, int trunkRadius, class_3341 boundingBox, Random rand) {
		for (int i = 0; i < (height * 0.83); i++) {
			method_12773(blocks, world, pos, tree.getLog(), boundingBox);
			pos.method_10098(class_2350.field_11036);
		}
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height, int bareTrunkHeight, int radius) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int i = 0; i < bareTrunkHeight; i++) {
			if (!method_16432(world, pos.method_10098(class_2350.field_11036))) {
				return false;
			}
		}

		for (int dY = bareTrunkHeight; dY < height; dY++) {
			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		return true;
	}

	public int getLeafHeight(Random rand) {
		return rand.nextInt(12) + 32;
	}

	public int getBareTrunkHeight(Random rand) {
		return 8 + rand.nextInt(12);
	}

	public int getMaxLeafRadius(Random rand) {
		return 6 + rand.nextInt(4);
	}

	public int getLeafLayers(Random rand) {
		return rand.nextInt(4) + 4;
	}

	public double getShrinkAmount() {
		return 1.0; //1.0 for 1 block
	}

	public int getMaxTrunkRadius(Random rand) {
		return 1;
	}
}
