package com.terraformersmc.terrestria.init;

import com.terraformersmc.terrestria.block.BasaltFlowerBlock;
import com.terraformersmc.terraform.block.*;
import com.terraformersmc.terraform.util.TerraformLargeSaplingGenerator;
import com.terraformersmc.terraform.util.TerraformSaplingGenerator;
import com.terraformersmc.terrestria.init.helpers.StoneBlocks;
import com.terraformersmc.terrestria.init.helpers.TerrestriaRegistry;
import com.terraformersmc.terrestria.init.helpers.WoodBlocks;
import com.terraformersmc.terrestria.init.helpers.WoodColors;
import net.fabricmc.fabric.api.block.FabricBlockSettings;
import net.fabricmc.fabric.api.registry.FlammableBlockRegistry;
import net.minecraft.block.*;
import net.minecraft.class_1294;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2261;
import net.minecraft.class_2362;
import net.minecraft.class_2397;
import net.minecraft.class_2468;
import net.minecraft.class_2476;
import net.minecraft.class_2525;
import net.minecraft.class_3084;
import net.minecraft.class_3111;
import net.minecraft.class_3620;

// This class exports public block constants, these fields have to be public
@SuppressWarnings("WeakerAccess")
public class TerrestriaBlocks {
	public static WoodBlocks REDWOOD;
	public static WoodBlocks HEMLOCK;
	public static WoodBlocks RUBBER;
	public static WoodBlocks CYPRESS;
	public static WoodBlocks WILLOW;
	public static WoodBlocks JAPANESE_MAPLE;
	public static WoodBlocks RAINBOW_EUCALYPTUS;
	public static WoodBlocks SAKURA;

	public static class_2397 JAPANESE_MAPLE_SHRUB_LEAVES;
	public static class_2397 DARK_JAPANESE_MAPLE_LEAVES;
	public static class_2397 JUNGLE_PALM_LEAVES;
	public static LeafPileBlock SAKURA_LEAF_PILE;

	public static class_2476 CATTAIL;
	public static class_2525 TALL_CATTAIL;

	public static QuarterLogBlock REDWOOD_QUARTER_LOG;
	public static QuarterLogBlock HEMLOCK_QUARTER_LOG;
	public static QuarterLogBlock CYPRESS_QUARTER_LOG;
	public static QuarterLogBlock RAINBOW_EUCALYPTUS_QUARTER_LOG;
	public static QuarterLogBlock STRIPPED_REDWOOD_QUARTER_LOG;
	public static QuarterLogBlock STRIPPED_HEMLOCK_QUARTER_LOG;
	public static QuarterLogBlock STRIPPED_CYPRESS_QUARTER_LOG;
	public static QuarterLogBlock STRIPPED_RAINBOW_EUCALYPTUS_QUARTER_LOG;

	public static TerraformSaplingBlock REDWOOD_SAPLING;
	public static TerraformSaplingBlock HEMLOCK_SAPLING;
	public static TerraformSaplingBlock RUBBER_SAPLING;
	public static TerraformSaplingBlock CYPRESS_SAPLING;
	public static TerraformSaplingBlock WILLOW_SAPLING;
	public static TerraformSaplingBlock JAPANESE_MAPLE_SAPLING;
	public static TerraformSaplingBlock JAPANESE_MAPLE_SHRUB_SAPLING;
	public static TerraformSaplingBlock DARK_JAPANESE_MAPLE_SAPLING;
	public static TerraformSaplingBlock RAINBOW_EUCALYPTUS_SAPLING;
	public static TerraformSaplingBlock SAKURA_SAPLING;
	public static TerraformSaplingBlock JUNGLE_PALM_SAPLING;

	public static class_2362 POTTED_REDWOOD_SAPLING;
	public static class_2362 POTTED_HEMLOCK_SAPLING;
	public static class_2362 POTTED_JUNGLE_PALM_SAPLING;
	public static class_2362 POTTED_RUBBER_SAPLING;
	public static class_2362 POTTED_CYPRESS_SAPLING;
	public static class_2362 POTTED_WILLOW_SAPLING;
	public static class_2362 POTTED_JAPANESE_MAPLE_SAPLING;
	public static class_2362 POTTED_JAPANESE_MAPLE_SHRUB_SAPLING;
	public static class_2362 POTTED_DARK_JAPANESE_MAPLE_SAPLING;
	public static class_2362 POTTED_RAINBOW_EUCALYPTUS_SAPLING;
	public static class_2362 POTTED_SAKURA_SAPLING;

	// Volcanic Island blocks
	public static class_2468 BASALT_SAND;
	public static class_2248 BASALT_DIRT;
	public static class_2248 BASALT_GRASS_BLOCK;
	public static StoneBlocks BASALT;
	public static class_2261 INDIAN_PAINTBRUSH;
	public static class_2261 MONSTERAS;
	public static class_2362 POTTED_INDIAN_PAINTBRUSH;
	public static class_2362 POTTED_MONSTERAS;

	public static void init() {
		FlammableBlockRegistry flammable = FlammableBlockRegistry.getDefaultInstance();

		REDWOOD = WoodBlocks.register("redwood", WoodColors.REDWOOD, flammable, true);
		HEMLOCK = WoodBlocks.register("hemlock", WoodColors.HEMLOCK, flammable, true);
		RUBBER = WoodBlocks.register("rubber", WoodColors.RUBBER, flammable);
		CYPRESS = WoodBlocks.register("cypress", WoodColors.CYPRESS, flammable);
		WILLOW = WoodBlocks.register("willow", WoodColors.WILLOW, flammable);
		JAPANESE_MAPLE = WoodBlocks.register("japanese_maple", WoodColors.JAPANESE_MAPLE, flammable);
		RAINBOW_EUCALYPTUS = WoodBlocks.register("rainbow_eucalyptus", WoodColors.RAINBOW_EUCALYPTUS, flammable);
		SAKURA = WoodBlocks.register("sakura", WoodColors.SAKURA, flammable, WoodBlocks.LogSize.SMALL);

		JAPANESE_MAPLE_SHRUB_LEAVES = TerrestriaRegistry.register("japanese_maple_shrub_leaves", new class_2397(class_2248.class_2251.method_9630(class_2246.field_10503)));

		DARK_JAPANESE_MAPLE_LEAVES = TerrestriaRegistry.register("dark_japanese_maple_leaves",
				new class_2397(FabricBlockSettings.copy(class_2246.field_10503).materialColor(class_3620.field_15982).build())
		);

		JUNGLE_PALM_LEAVES = TerrestriaRegistry.register("jungle_palm_leaves", new class_2397(class_2248.class_2251.method_9630(class_2246.field_10503)));

		SAKURA_LEAF_PILE = TerrestriaRegistry.register("sakura_leaf_pile", new LeafPileBlock(class_2248.class_2251.method_9630(SAKURA.leaves).method_9634()));

		flammable.add(JAPANESE_MAPLE_SHRUB_LEAVES, 30, 60);
		flammable.add(DARK_JAPANESE_MAPLE_LEAVES, 30, 60);
		flammable.add(JUNGLE_PALM_LEAVES, 30, 60);
		flammable.add(SAKURA_LEAF_PILE, 30, 60);

		TALL_CATTAIL = TerrestriaRegistry.register("tall_cattail", new TallCattailBlock(() -> TerrestriaItems.CATTAIL, class_2248.class_2251.method_9630(class_2246.field_10376)));
		CATTAIL = TerrestriaRegistry.register("cattail", new TerraformSeagrassBlock(TALL_CATTAIL, class_2248.class_2251.method_9630(class_2246.field_10376)));

		REDWOOD_QUARTER_LOG = REDWOOD.registerQuarterLog(() -> STRIPPED_REDWOOD_QUARTER_LOG, flammable);
		HEMLOCK_QUARTER_LOG = HEMLOCK.registerQuarterLog(() -> STRIPPED_HEMLOCK_QUARTER_LOG, flammable);
		CYPRESS_QUARTER_LOG = CYPRESS.registerQuarterLog(() -> STRIPPED_CYPRESS_QUARTER_LOG, flammable);
		RAINBOW_EUCALYPTUS_QUARTER_LOG = RAINBOW_EUCALYPTUS.registerQuarterLog(() -> STRIPPED_RAINBOW_EUCALYPTUS_QUARTER_LOG, flammable);
		STRIPPED_REDWOOD_QUARTER_LOG = REDWOOD.registerStrippedQuarterLog(flammable);
		STRIPPED_HEMLOCK_QUARTER_LOG = HEMLOCK.registerStrippedQuarterLog(flammable);
		STRIPPED_CYPRESS_QUARTER_LOG = CYPRESS.registerStrippedQuarterLog(flammable);
		STRIPPED_RAINBOW_EUCALYPTUS_QUARTER_LOG = RAINBOW_EUCALYPTUS.registerStrippedQuarterLog(flammable);

		// Saplings

		REDWOOD_SAPLING = TerrestriaRegistry.register("redwood_sapling", new TerraformSaplingBlock(
				new TerraformLargeSaplingGenerator(
						() -> TerrestriaFeatures.REDWOOD_TREE.sapling(),
						() -> TerrestriaFeatures.MEGA_REDWOOD_TREE.sapling()
				)
		));

		HEMLOCK_SAPLING = TerrestriaRegistry.register("hemlock_sapling", new TerraformSaplingBlock(
				new TerraformLargeSaplingGenerator(
						() -> TerrestriaFeatures.HEMLOCK_TREE.sapling(),
						() -> TerrestriaFeatures.MEGA_HEMLOCK_TREE.sapling()
				)
		));

		RUBBER_SAPLING = TerrestriaRegistry.register("rubber_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(
						() -> TerrestriaFeatures.RUBBER_TREE.sapling()
				)
		));

		CYPRESS_SAPLING = TerrestriaRegistry.register("cypress_sapling", new TerraformSaplingBlock(
				new TerraformLargeSaplingGenerator(
						() -> TerrestriaFeatures.CYPRESS_TREE.sapling(),
						() -> TerrestriaFeatures.MEGA_CYPRESS_TREE.sapling()
				)
		));

		WILLOW_SAPLING = TerrestriaRegistry.register("willow_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(() -> TerrestriaFeatures.WILLOW_TREE.sapling())
		));

		JAPANESE_MAPLE_SAPLING = TerrestriaRegistry.register("japanese_maple_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(() -> TerrestriaFeatures.JAPANESE_MAPLE_TREE.sapling())
		));

		JAPANESE_MAPLE_SHRUB_SAPLING = TerrestriaRegistry.register("japanese_maple_shrub_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(() -> TerrestriaFeatures.JAPANESE_MAPLE_SHRUB.sapling())
		));

		DARK_JAPANESE_MAPLE_SAPLING = TerrestriaRegistry.register("dark_japanese_maple_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(() -> TerrestriaFeatures.DARK_JAPANESE_MAPLE_TREE.sapling())
		));

		RAINBOW_EUCALYPTUS_SAPLING = TerrestriaRegistry.register("rainbow_eucalyptus_sapling", new TerraformSaplingBlock(
				new TerraformLargeSaplingGenerator(
						() -> new class_3084(class_3111::method_13565, true, 5,
								TerrestriaBlocks.RAINBOW_EUCALYPTUS.log.method_9564(),
								TerrestriaBlocks.RAINBOW_EUCALYPTUS.leaves.method_9564(),
								true
						),
						() -> TerrestriaFeatures.RAINBOW_EUCALYPTUS_TREE.sapling()
				)
		));

		SAKURA_SAPLING = TerrestriaRegistry.register("sakura_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(() -> TerrestriaFeatures.SAKURA_TREE.sapling())
		));

		JUNGLE_PALM_SAPLING = TerrestriaRegistry.register("jungle_palm_sapling", new TerraformSaplingBlock(
				new TerraformSaplingGenerator(() -> TerrestriaFeatures.JUNGLE_PALM_TREE.sapling())
		));

		// Volcanic Island Blocks

		BASALT_SAND = TerrestriaRegistry.register("basalt_sand", new class_2468(0x202020, FabricBlockSettings.copy(class_2246.field_10102).materialColor(class_3620.field_16009).build()));
		BASALT_DIRT = TerrestriaRegistry.register("basalt_dirt", new class_2248(FabricBlockSettings.copy(class_2246.field_10566).materialColor(class_3620.field_16009).build()));
		BASALT_GRASS_BLOCK = TerrestriaRegistry.register("basalt_grass_block", new TerraformGrassBlock(BASALT_DIRT, class_2248.class_2251.method_9630(class_2246.field_10219)));
		BASALT = StoneBlocks.register("basalt", class_3620.field_16009);

		INDIAN_PAINTBRUSH = TerrestriaRegistry.register("indian_paintbrush", new BasaltFlowerBlock(class_1294.field_5922, 6, class_2248.class_2251.method_9630(class_2246.field_10449)));
		MONSTERAS = TerrestriaRegistry.register("monsteras", new BasaltFlowerBlock(class_1294.field_5924, 2, class_2248.class_2251.method_9630(class_2246.field_10449)));

		POTTED_INDIAN_PAINTBRUSH = TerrestriaRegistry.register("potted_indian_paintbrush", new class_2362(INDIAN_PAINTBRUSH, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_MONSTERAS = TerrestriaRegistry.register("potted_monsteras", new class_2362(MONSTERAS, class_2248.class_2251.method_9630(class_2246.field_10151)));

		POTTED_REDWOOD_SAPLING = TerrestriaRegistry.register("potted_redwood_sapling", new class_2362(REDWOOD_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_HEMLOCK_SAPLING = TerrestriaRegistry.register("potted_hemlock_sapling", new class_2362(HEMLOCK_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_RUBBER_SAPLING = TerrestriaRegistry.register("potted_rubber_sapling", new class_2362(RUBBER_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_CYPRESS_SAPLING = TerrestriaRegistry.register("potted_cypress_sapling", new class_2362(CYPRESS_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_WILLOW_SAPLING = TerrestriaRegistry.register("potted_willow_sapling", new class_2362(WILLOW_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_JAPANESE_MAPLE_SAPLING = TerrestriaRegistry.register("potted_japanese_maple_sapling", new class_2362(JAPANESE_MAPLE_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_JAPANESE_MAPLE_SHRUB_SAPLING = TerrestriaRegistry.register("potted_japanese_maple_shrub_sapling", new class_2362(JAPANESE_MAPLE_SHRUB_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_DARK_JAPANESE_MAPLE_SAPLING = TerrestriaRegistry.register("potted_dark_japanese_maple_sapling", new class_2362(DARK_JAPANESE_MAPLE_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_RAINBOW_EUCALYPTUS_SAPLING = TerrestriaRegistry.register("potted_rainbow_eucalyptus_sapling", new class_2362(RAINBOW_EUCALYPTUS_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_SAKURA_SAPLING = TerrestriaRegistry.register("potted_sakura_sapling", new class_2362(SAKURA_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
		POTTED_JUNGLE_PALM_SAPLING = TerrestriaRegistry.register("potted_jungle_palm_sapling", new class_2362(JUNGLE_PALM_SAPLING, class_2248.class_2251.method_9630(class_2246.field_10151)));
	}
}
